import { callbackify } from 'util'

import through from 'through2-concurrent'

import { parseOpts } from '../options/main.js'
import { throwError, handleError } from '../error.js'

import { getDefaultOpts, forcedOpts } from './options.js'
import { setResult } from './result.js'

// Creates a stream that fires child processes on each file:
//   gulp.src(...).pipe(stream(({ path }) => `command ${path}`))
export const stream = function(getInput, opts) {
  const eGetInput = handleGetInput({ getInput })

  const defaultOpts = getDefaultOpts({ opts })
  const { maxConcurrency, result: resultOpt, ...optsA } = parseOpts({
    opts,
    defaultOpts,
    forcedOpts,
  })

  return through.obj(
    { maxConcurrency },
    execVinyl.bind(null, { getInput: eGetInput, opts: optsA, resultOpt }),
  )
}

const handleGetInput = function({ getInput }) {
  if (typeof getInput !== 'function') {
    throwError(`Option 'getInput' must be a function: ${getInput}`)
  }

  return handleError(getInput)
}

const cExecVinyl = async function({ getInput, opts, resultOpt }, file) {
  const input = await getInput(file)

  await setResult({ file, input, opts, resultOpt })

  return file
}

const execVinyl = callbackify(cExecVinyl)
